/*
    This file is part of Funiter,
    Real and complex function iteration software.

    main.c - Funiter's main routine.

    Copyright (C) 1995-2006 Stijn Wolters.
    Original idea: Ernic Kamerich (University of Nijmegen).

    See README for contact information.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    See COPYING for more information.
*/

#include <stdio.h>
#include <allegro.h>

#ifdef BUILD_WITH_ADIME
#include <adime.h>
#endif

#ifdef BUILD_WITH_AGUP
#include <agup.h>
#endif

#include "newmenus.h"
#include "clsmenus.h"
#include "realexp.h"
#include "cplexp.h"

int Progress(int Position, int MaxPosition);

int main(int argc, char *argv[])
{
    char        LngFile[2048] = "english.mnu", AGUP_Theme[1024] = "Photon";
    int         i, card, width = 640, height = 480, color_depth,
                NewMenus = FALSE;
    TConsole    *Console;
    TRealExp    *RealExp;
    TComplexExp *ComplexExp;

    /* Initialize graphics */
    
    set_uformat(U_ASCII);
    
    allegro_init();
    install_mouse();
    install_keyboard();
    install_timer();
    if(set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0) != 0) {
        if(set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) { 
            set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
            allegro_message("Error setting a graphmode!");
            exit(1);
        }
    }

#ifdef BUILD_WITH_ADIME
    adime_init();
#endif

#ifdef BUILD_WITH_AGUP
    agup_init(aphoton_theme);
#endif
    
    /* Look for commandline options. */
    
    for(i = 1; i < argc; i++) {

        /* Load language file */
        
        if(strncmp(argv[i], "-l", 2) == 0) {
            strcpy(LngFile, argv[i] + 2);
        }
        
        /* Select graphic-mode */
        
        else if(strncmp(argv[i], "-s", 2) == 0) {
            gfx_mode_select_ex(&card, &width, &height, &color_depth);
            if(set_gfx_mode(card, width, height, 0, 0) != 0) {
                set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
                allegro_message("Error setting a graphmode!");
                exit(1);
            }
        }
        /*
        else if(strncmp(argv[i], "-t", 2) == 0) {
            strcpy(AGUP_Theme, argv[i] + 2);
#ifdef BUILD_WITH_AGUP
            agup_shutdown();
            agup_init(agup_theme_by_name(AGUP_Theme));
#endif
        }
        */
        /* New menu-style */
        /*
        else if(strncmp(argv[i], "-n", 2) == 0)
            NewMenus = TRUE;
        */
        
        /* Help */
        
        else if((strncmp(argv[i], "-?", 2) == 0) ||
            (strncmp(argv[i], "-h", 2) == 0)) 
        {
            printf("\nUsage: %s [-s] [-l<filename>] [-? | -h]", argv[0]);
            printf("\n-l<filename> = load a language file");
            printf("\n-s           = open dialog to select a graphicmode");
            printf("\n-? or -h     = help");
            printf("\n\nCommandline parameters are lowercase only!\n");
            return 0;
        }
        else {
            printf("Unknown command, use -h or -? for help.\n");
            return 0;
        }
    }

    clear_keybuf();
    RealExp = CreateRealExp(0, 0, width, height - 20);
    RealExp->PrgProc = Progress;
    ComplexExp = CreateComplexExp(0, 0, width, height - 20);
    ComplexExp->PrgProc = Progress;
    /*
    if(NewMenus)
        NewMnuMain(RealExp, ComplexExp, LngFile);
    */
    if(!NewMenus) {
        Console = alconio_init(80, 30, 2, LIGHTGRAY, BLACK);
        InitClassicMenus(0, LngFile);
        ClsMnuMain(Console, RealExp, ComplexExp);
        alconio_exit(Console);
    }
    
#ifdef BUILD_WITH_ADIME
    adime_exit();
#endif
#ifdef BUILD_WITH_AGUP
    agup_shutdown();
#endif
    allegro_exit();

    FreeComplexExp(ComplexExp);
    FreeRealExp(RealExp);
    return 0;
}
END_OF_MAIN()

int Progress(int Position, int MaxPosition)
{
    int         Percentage = 0;
    double      OnePercent = 1.0;
    BITMAP      *Bitmap;
    
    OnePercent = (double) MaxPosition / 100.0;
    if(Position % (int) OnePercent == 0) {
        Percentage = ((double) Position / OnePercent);
        Bitmap = create_bitmap(110, 30);
        clear_bitmap(Bitmap);
        acquire_bitmap(Bitmap);
        rect(Bitmap, 0, 0, 109, 29, makecol(255, 255, 255));
        if(Percentage > 0) { 
            rectfill(Bitmap, 5, 5, 5 + Percentage - 1, 25, 
                makecol(110, 110, 110));
        }
        textprintf_ex(Bitmap, font, 45, (30 - text_height(font)) / 2,
            makecol(255, 255, 255), -1, "%3d%%", Percentage);
        release_bitmap(Bitmap);
        blit(Bitmap, screen, 0, 0, (SCREEN_W - 109) / 2, 
            (SCREEN_H - 29) / 2, 110, 30);
        destroy_bitmap(Bitmap);
    }
    return 0;
}
